package Bio::Palantir::Parser::Qualifier;
# ABSTRACT: BiosynML DTD-derived internal class
$Bio::Palantir::Parser::Qualifier::VERSION = '0.191800';
use Moose;
use namespace::autoclean;

# AUTOGENERATED CODE! DO NOT MODIFY THIS FILE!

use XML::Bare qw(forcearray);



# private attributes
has '_root' => (
    is       => 'ro',
    isa      => 'HashRef',
    required => 1,
);



# public array(s) of composed objects


# public composed object(s)


# public deep methods


# public methods



# public aliases


sub name {
    return shift->_root->{'name'}->{'value'}
} 


sub ori {
    return shift->_root->{'ori'}->{'value'}
} 


sub style {
    return shift->_root->{'style'}->{'value'}
} 


sub value {
    return shift->_root->{'value'}
} 


__PACKAGE__->meta->make_immutable;
1;

__END__

=pod

=head1 NAME

Bio::Palantir::Parser::Qualifier - BiosynML DTD-derived internal class

=head1 VERSION

version 0.191800

=head1 SYNOPSIS

    # TODO

=head1 DESCRIPTION

    # TODO

=head1 METHODS

=head2 name

Returns the name attribute of the qualifier.

    # $qualifier is a Bio::FastParsers::Biosynml
    my $name = $qualifier->name;

This method does not accept any arguments.

=head2 ori

Returns the ori attribute of the qualifier.

    # $qualifier is a Bio::FastParsers::Biosynml
    my $ori = $qualifier->ori;

This method does not accept any arguments.

=head2 style

Returns the style attribute of the qualifier.

    # $qualifier is a Bio::FastParsers::Biosynml
    my $style = $qualifier->style;

This method does not accept any arguments.

=head2 value

Returns the value of the qualifier.

    # $qualifier is a Bio::FastParsers::Biosynml
    my $value = $qualifier->value;

This method does not accept any arguments.

=head1 AUTHOR

Loic MEUNIER <lmeunier@uliege.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2019 by University of Liege / Unit of Eukaryotic Phylogenomics / Loic MEUNIER and Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
